@@ -65,7 +65,7 @@ const std::string p4s = "+proj=longlat +datum=WGS84 +no_defs";
  *      4b. clear_arr ()
  *      4c. clean_vecs ()
  *      4d. clean_arrs ()
- * 4. osmdatacpp
+ * 4. osmdata.cpp
  *      5c. get_osm_relations ()
  *      5d. get_osm_ways ()
  *      5e. get_osm_nodes ()
@@ -116,12 +116,11 @@ class XmlData
         Relations m_relations;
         UniqueVals m_unique;
 
-    protected:
-
-        float xmin=FLOAT_MAX, xmax=-FLOAT_MAX, ymin=FLOAT_MAX, ymax=-FLOAT_MAX;
-
     public:
 
+        double xmin = DOUBLE_MAX, xmax = -DOUBLE_MAX,
+              ymin = DOUBLE_MAX, ymax = -DOUBLE_MAX;
+
         XmlData (const std::string& str)
         {
             // APS empty m_nodes/m_ways/m_relations constructed here, no need to explicitly clear
@@ -140,10 +139,10 @@ class XmlData
         const Ways& ways() const { return m_ways; }
         const Relations& relations() const { return m_relations; }
         const UniqueVals& unique_vals() const { return m_unique; }
-        const float x_min() { return xmin;  }
-        const float x_max() { return xmax;  }
-        const float y_min() { return ymin;  }
-        const float y_max() { return ymax;  }
+        double x_min() { return xmin;  }
+        double x_max() { return xmax;  }
+        double y_min() { return ymin;  }
+        double y_max() { return ymax;  }
 
     private:
 
@@ -387,9 +386,9 @@ inline void XmlData::traverseNode (XmlNodePtr pt, RawNode& rnode)
         if (!strcmp (it->name(), "id"))
             rnode.id = std::stoll(it->value());
         else if (!strcmp (it->name(), "lat"))
-            rnode.lat = std::stof(it->value());
+            rnode.lat = std::stod(it->value());
         else if (!strcmp (it->name(), "lon"))
-            rnode.lon = std::stof(it->value());
+            rnode.lon = std::stod(it->value());
         else if (!strcmp (it->name(), "k"))
             rnode.key.push_back (it->value ());
         else if (!strcmp (it->name(), "v"))
@@ -401,3 +400,31 @@ inline void XmlData::traverseNode (XmlNodePtr pt, RawNode& rnode)
         traverseNode (it, rnode);
     }
 } // end function XmlData::traverseNode
+
+
+/*---------------------------- fn headers -----------------------------*/
+// in osmdata_sf.cpp
+
+Rcpp::List get_osm_relations_sf (const Relations &rels, 
+        const std::map <osmid_t, Node> &nodes,
+        const std::map <osmid_t, OneWay> &ways, const UniqueVals &unique_vals,
+        const Rcpp::NumericVector &bbox, const Rcpp::List &crs);
+void get_osm_ways_sf (Rcpp::List &wayList, Rcpp::DataFrame &kv_df,
+        const std::set <osmid_t> way_ids, const Ways &ways, const Nodes &nodes,
+        const UniqueVals &unique_vals, const std::string &geom_type,
+        const Rcpp::NumericVector &bbox, const Rcpp::List &crs);
+void get_osm_nodes_sf (Rcpp::List &ptList, Rcpp::DataFrame &kv_df,
+        const Nodes &nodes, const UniqueVals &unique_vals, 
+        const Rcpp::NumericVector &bbox, const Rcpp::List &crs);
+Rcpp::List rcpp_osmdata_sf (const std::string& st);
+
+// in osmdata_sp.cpp
+void get_osm_nodes_sp (Rcpp::S4 &sp_points, const Nodes &nodes, 
+        const UniqueVals &unique_vals);
+void get_osm_ways_sp (Rcpp::S4 &sp_ways, 
+        const std::set <osmid_t> way_ids, const Ways &ways, const Nodes &nodes,
+        const UniqueVals &unique_vals, const std::string &geom_type);
+void get_osm_relations_sp (Rcpp::S4 &multilines, Rcpp::S4 &multipolygons, 
+        const Relations &rels, const std::map <osmid_t, Node> &nodes,
+        const std::map <osmid_t, OneWay> &ways, const UniqueVals &unique_vals);
+Rcpp::List rcpp_osmdata_sp (const std::string& st);
